<?php

namespace App\Controller;

use App\Entity\Product;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Response;

class ProductController extends Controller
{

  /**
   * @Route(
   *  name = "product_index",
   *  path = "/product"
   * )
   *
   * @param \Doctrine\ORM\EntityManagerInterface $em
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function index(EntityManagerInterface $em) {
    $repository = $em->getRepository(Product::class);
    $products = $repository->findAll();
    return $this->render('product/index.html.twig', ['products' => $products]);
  }

  /**
   * @Route(
   *   name="product_add",
   *   path = "/product/add"
   * )
   */
  public function add() {
    // EntityManager can also be injected as EntityManagerInterface $em.
    $em = $this->getDoctrine()->getManager();

    $product = new Product();
    $product->setName('Keyboard')
      ->setPrice(19.99)
      ->setDescription('Ergonomic and stylish!');

    // tell Doctrine you want to (eventually) save the Product (no queries yet)
    $em->persist($product);

    // actually executes the queries (i.e. the INSERT query)
    $em->flush();

    return new Response('Saved new product with id ' . $product->getId(),
      Response::HTTP_CREATED);
  }

  /**
   * @Route(
   *   name = "product_show_manual",
   *   path = "/product/{id}/manual",
   * )
   *
   * @param $id
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function showManually($id) {
    $repository = $this->getDoctrine()
      ->getRepository(Product::class);

    // Find by PK (id).
    $product = $repository->find($id);

    // Find single entity by properties
    $product = $repository->findOneBy([
      'name' => 'Keyboard',
    ]);

    $productsBy = $repository->findBy([
      'name' => 'Keyboard',
    ]);
    $this->addFlash('info', array_reduce($productsBy, function (string $accu, Product $product) {
      return $accu . '/' .  $product->getId() . ' ' . $product->getName() . ': ' . $product->getDescription();
    }, ''));

    $productsAll = $repository->findAll();
    $this->addFlash('error', array_reduce($productsAll, function (string $accu, Product $product) {
      return $accu . '/' .  $product->getId() . ' ' . $product->getName() . ': ' . $product->getDescription();
    }, ''));

    if (!$product) {
      throw $this->createNotFoundException(
        "No product found for id $id"
      );
    }

    return $this->render('product/product.html.twig', [
      'product' => $product,
    ]);
  }

  /**
   * @Route(
   *   name = "product_show",
   *   path = "/product/{id}",
   * )
   *
   * @param \App\Entity\Product $product
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function show(Product $product) {
    return $this->render('product/product.html.twig', [
      'product' => $product,
    ]);
  }

  /**
   * @Route(
   *   name = "product_raise",
   *   path = "/product/{id}/raise"
   * )
   *
   * @param \App\Entity\Product $product
   * @param \Doctrine\ORM\EntityManagerInterface $em
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function update(Product $product, EntityManagerInterface $em) {
    $price = $product->getPrice();
    $price += 0.01;
    $product->setPrice($price);
    // $em->persist($product) is NOT needed but MAY be used as product is
    // already mapped.
    $em->flush();

    $id = $product->getId();
    $request = Request::create($this->generateUrl('product_show', ['id' => $id]));
    $kernel = $this->get('http_kernel');
    return $kernel->handle($request);
  }

  /**
   * @Route(
   *   name = "product_delete",
   *   path = "/product/{id}/delete"
   * )
   *
   * @param \App\Entity\Product $product
   * @param \Doctrine\ORM\EntityManagerInterface $em
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function delete(Product $product, EntityManagerInterface $em) {
    $em->remove($product);
    $em->flush();

    return $this->redirectToRoute('product_index');
  }
}
